searchData={"items":[{"type":"module","title":"ExTermbox.Bindings","doc":"Provides the low-level bindings to the termbox library. This module loads the\nNIFs defined in `c_src/` and thinly wraps the C interface.\n\nFor event-handling, it's recommended to use the `ExTermbox.EventManager` API\ninstead of the raw interface exposed here.\n\nFor more complex applications, it's recommended to use the high-level\nrendering API provided by Ratatouille (a terminal UI kit based on the bindings\nhere). Ratatouille manages things like initialization, updates and shutdown\nautomatically, and provides a declarative, HTML-like interface for rendering\ncontent to the screen. See the repo for details:\n\n \n\nSee also the termbox header file for additional documentation of the functions\nhere:\n\n \n\nNote that the \"NIF  /  not loaded\" messages below are fallbacks\nnormally replaced by the natively-implemented functions at load. If you're\nseeing this message, it means the native bindings could not be loaded. Please\nopen an issue with the error and relevant system information.","ref":"ExTermbox.Bindings.html"},{"type":"module","title":"Event Polling - ExTermbox.Bindings","doc":"The event polling API differs slightly from the termbox API in order to make\nit in the Erlang ecosystem. Instead of blocking poll calls, it uses\nasynchronous message passing to deliver events to the caller.\n\nIt's recommended to use the `ExTermbox.EventManager` gen_server to subscribe\nto terminal events instead of using these bindings directly. It supports\nmultiple subscriptions and more gracefully handles errors.\n\n#### Implementation Notes\n\nIn the `start_polling/1` NIF, an OS-level thread is created which performs the\nblocking event polling (i.e., a `select` call). This allows the NIF to return\nquickly and avoid causing the scheduler too much trouble. It would be very bad\nto block the scheduler thread until an event is received.\n\nWhile using threads solves this problem, it unfortunately also introduces new\nones. The bindings implement some locking mechanisms to try to coordinate\nthreading logic and prevent polling from occurring simultaneously, but this\nsort of logic is hard to get right (one of the reasons we use Elixir/Erlang).\nNo issues are currently known, but please report any you happen to encounter.\n\n#### Timeouts\n\nYou might have noticed that there's no binding for `tb_peek_event` (which\naccepts a timeout). That's because it's easy enough to implement a timeout\nourselves with `start_polling/1` and `receive` with `after`, e.g.:\n\n    {:ok, _resource} = Bindings.start_polling(self())\n\n    receive do\n      {:event, event} ->\n        # handle the event...\n    after\n      1_000 ->\n        :ok = Bindings.stop_polling(self())\n        # do something else...\n    end","ref":"ExTermbox.Bindings.html#module-event-polling"},{"type":"function","title":"ExTermbox.Bindings.change_cell/5","doc":"Changes the attributes of the cell at the specified position in the internal\nback buffer. Prefer using `put_cell/1`, which supports passing an\n`ExTermbox.Cell` struct.\n\nReturns `:ok` if successful. If termbox was not initialized, returns\n`{:error, :not_running}` (call `init/0` first).","ref":"ExTermbox.Bindings.html#change_cell/5"},{"type":"function","title":"ExTermbox.Bindings.clear/0","doc":"Clears the internal back buffer, setting the foreground and background to the\ndefaults, or those specified by `set_clear_attributes/2`.\n\nReturns `:ok` if successful. If termbox was not initialized, returns\n`{:error, :not_running}` (call `init/0` first).","ref":"ExTermbox.Bindings.html#clear/0"},{"type":"function","title":"ExTermbox.Bindings.height/0","doc":"Returns `{:ok, height}` where `height` is the height of the terminal window in\ncharacters.\n\nIf termbox was not initialized, returns `{:error, :not_running}` (call\n`init/0` first).","ref":"ExTermbox.Bindings.html#height/0"},{"type":"function","title":"ExTermbox.Bindings.init/0","doc":"Initializes the termbox library. Must be called before any other bindings are\ncalled.\n\nReturns `:ok` on success and otherwise one of the following errors:\n\n* `{:error, :already_running} - the library was already initialized.\n* `{:error, code}` - where code is an integer error code from termbox.","ref":"ExTermbox.Bindings.html#init/0"},{"type":"function","title":"ExTermbox.Bindings.load_nifs/0","doc":"","ref":"ExTermbox.Bindings.html#load_nifs/0"},{"type":"function","title":"ExTermbox.Bindings.nif_content/1","doc":"","ref":"ExTermbox.Bindings.html#nif_content/1"},{"type":"function","title":"ExTermbox.Bindings.nif_path/1","doc":"","ref":"ExTermbox.Bindings.html#nif_path/1"},{"type":"function","title":"ExTermbox.Bindings.present/0","doc":"Synchronizes the internal back buffer and the terminal.\n\nReturns `:ok` if successful. If termbox was not initialized, returns\n`{:error, :not_running}` (call `init/0` first).","ref":"ExTermbox.Bindings.html#present/0"},{"type":"function","title":"ExTermbox.Bindings.put_cell/1","doc":"Puts a cell in the internal back buffer at the cell's position. Note that this is\nimplemented in terms of `change_cell/5`.\n\nReturns `:ok` if successful. If termbox was not initialized, returns\n`{:error, :not_running}` (call `init/0` first).","ref":"ExTermbox.Bindings.html#put_cell/1"},{"type":"function","title":"ExTermbox.Bindings.select_input_mode/1","doc":"Sets or retrieves the input mode (see `ExTermbox.Constants.input_modes/0`).\nSee the [termbox source](https://github.com/nsf/termbox/blob/master/src/termbox.h)\nfor additional documentation.\n\nReturns `{:ok, input_mode}` when successful, where `input_mode` is an integer\nrepresenting the current mode. If termbox was not initialized, returns\n`{:error, :not_running}` (call `init/0` first).","ref":"ExTermbox.Bindings.html#select_input_mode/1"},{"type":"function","title":"ExTermbox.Bindings.select_output_mode/1","doc":"Sets or retrieves the output mode (see `ExTermbox.Constants.output_modes/0`).\nSee the [termbox source](https://github.com/nsf/termbox/blob/master/src/termbox.h)\nfor additional documentation.\n\nReturns `{:ok, output_mode}` when successful, where `output_mode` is an\ninteger representing the current mode. If termbox was not initialized, returns\n`{:error, :not_running}` (call `init/0` first).","ref":"ExTermbox.Bindings.html#select_output_mode/1"},{"type":"function","title":"ExTermbox.Bindings.set_clear_attributes/2","doc":"Sets the default foreground and background colors used when `clear/0` is\ncalled.\n\nReturns `:ok` if successful. If termbox was not initialized, returns\n`{:error, :not_running}` (call `init/0` first).","ref":"ExTermbox.Bindings.html#set_clear_attributes/2"},{"type":"function","title":"ExTermbox.Bindings.set_cursor/2","doc":"Sets the position of the cursor to the coordinates `(x, y)`, or hide the\ncursor by passing `ExTermbox.Constants.hide_cursor/0` for both x and y.\n\nReturns `:ok` if successful. If termbox was not initialized, returns\n`{:error, :not_running}` (call `init/0` first).","ref":"ExTermbox.Bindings.html#set_cursor/2"},{"type":"function","title":"ExTermbox.Bindings.shutdown/0","doc":"Finalizes the termbox library. Should be called when the terminal application\nis exited, and before your program or OTP application stops.\n\nReturns `:ok` on success and otherwise one of the following errors:\n\n* `{:error, :not_running} - the library can not be shut down because it is not\n  initialized.\n* `{:error, code}` - where `code` is an integer error code from termbox.","ref":"ExTermbox.Bindings.html#shutdown/0"},{"type":"function","title":"ExTermbox.Bindings.start_polling/1","doc":"Starts polling for terminal events asynchronously. The function accepts a PID\nas argument and returns immediately. When an event is received, it's sent to\nthe specified process. It continues polling until either `stop_polling/0` or\n`shutdown/0` is called. An error is returned when this function is called\nagain before polling has been stopped.\n\nIf successful, returns `{:ok, resource}`, where `resource` is an Erlang\nresource object representing a handle for the poll thread. Otherwise, one of\nthe following errors is returned:\n\n* `{:error, :not_running} - termbox should be initialized before events are\n  polled.\n* `{:error, :already_polling}` - `start_polling/1` was previously called and\n  has not been since stopped.","ref":"ExTermbox.Bindings.html#start_polling/1"},{"type":"function","title":"ExTermbox.Bindings.stop_polling/0","doc":"Cancels a previous call to `start_polling/1` and blocks until polling has\nstopped. The polling loop checks every 10 ms for a stop condition, so calls\ncan take up to 10 ms to return.\n\nThis can be useful, for example, if the `start_polling/1` recipient process\ndies and the polling needs to be restarted by another process.\n\nReturns `:ok` on success and otherwise one of the following errors:\n\n* `{:error, :not_running} - termbox should be initialized before any polling\n  functions are called.\n* `{:error, :not_polling} - polling cannot be stopped because it was already\n  stopped or never started.","ref":"ExTermbox.Bindings.html#stop_polling/0"},{"type":"function","title":"ExTermbox.Bindings.width/0","doc":"Returns `{:ok, width}` where `width` is the width of the terminal window in\ncharacters.\n\nIf termbox was not initialized, returns `{:error, :not_running}` (call\n`init/0` first).","ref":"ExTermbox.Bindings.html#width/0"},{"type":"module","title":"ExTermbox.Cell","doc":"Represents a termbox cell, a character at a position, along with the cell's\nbackground and foreground colors.","ref":"ExTermbox.Cell.html"},{"type":"function","title":"ExTermbox.Cell.empty/0","doc":"","ref":"ExTermbox.Cell.html#empty/0"},{"type":"type","title":"ExTermbox.Cell.t/0","doc":"","ref":"ExTermbox.Cell.html#t:t/0"},{"type":"module","title":"ExTermbox.Constants","doc":"Defines constants from the termbox library. These can be used e.g. to set a\nformatting attributes or to identify keys passed in an event.","ref":"ExTermbox.Constants.html"},{"type":"type","title":"ExTermbox.Constants.attribute/0","doc":"","ref":"ExTermbox.Constants.html#t:attribute/0"},{"type":"function","title":"ExTermbox.Constants.attribute/1","doc":"Retrieves an attribute constant by name","ref":"ExTermbox.Constants.html#attribute/1"},{"type":"function","title":"Examples - ExTermbox.Constants.attribute/1","doc":"iex> attribute(:bold)\n    0x0100\n    iex> attribute(:underline)\n    0x0200","ref":"ExTermbox.Constants.html#attribute/1-examples"},{"type":"function","title":"ExTermbox.Constants.attributes/0","doc":"Retrieves the mapping of attribute constants.","ref":"ExTermbox.Constants.html#attributes/0"},{"type":"type","title":"ExTermbox.Constants.color/0","doc":"","ref":"ExTermbox.Constants.html#t:color/0"},{"type":"function","title":"ExTermbox.Constants.color/1","doc":"Retrieves a color constant by name","ref":"ExTermbox.Constants.html#color/1"},{"type":"function","title":"Examples - ExTermbox.Constants.color/1","doc":"iex> color(:red)\n    0x02\n    iex> color(:blue)\n    0x05","ref":"ExTermbox.Constants.html#color/1-examples"},{"type":"function","title":"ExTermbox.Constants.colors/0","doc":"Retrieves the mapping of color constants.","ref":"ExTermbox.Constants.html#colors/0"},{"type":"type","title":"ExTermbox.Constants.constant/0","doc":"","ref":"ExTermbox.Constants.html#t:constant/0"},{"type":"type","title":"ExTermbox.Constants.error_code/0","doc":"","ref":"ExTermbox.Constants.html#t:error_code/0"},{"type":"function","title":"ExTermbox.Constants.error_code/1","doc":"Retrieves an error code constant by name","ref":"ExTermbox.Constants.html#error_code/1"},{"type":"function","title":"Examples - ExTermbox.Constants.error_code/1","doc":"iex> error_code(:unsupported_terminal)\n    -1","ref":"ExTermbox.Constants.html#error_code/1-examples"},{"type":"function","title":"ExTermbox.Constants.error_codes/0","doc":"Retrieves the mapping of error code constants.","ref":"ExTermbox.Constants.html#error_codes/0"},{"type":"type","title":"ExTermbox.Constants.event_type/0","doc":"","ref":"ExTermbox.Constants.html#t:event_type/0"},{"type":"function","title":"ExTermbox.Constants.event_type/1","doc":"Retrieves an event type constant by name","ref":"ExTermbox.Constants.html#event_type/1"},{"type":"function","title":"Examples - ExTermbox.Constants.event_type/1","doc":"iex> event_type(:key)\n    0x01\n    iex> event_type(:resize)\n    0x02\n    iex> event_type(:mouse)\n    0x03","ref":"ExTermbox.Constants.html#event_type/1-examples"},{"type":"function","title":"ExTermbox.Constants.event_types/0","doc":"Retrieves the mapping of event type constants.","ref":"ExTermbox.Constants.html#event_types/0"},{"type":"function","title":"ExTermbox.Constants.hide_cursor/0","doc":"Retrieves the hide cursor constant.","ref":"ExTermbox.Constants.html#hide_cursor/0"},{"type":"function","title":"Examples - ExTermbox.Constants.hide_cursor/0","doc":"iex> hide_cursor()\n    -1","ref":"ExTermbox.Constants.html#hide_cursor/0-examples"},{"type":"type","title":"ExTermbox.Constants.hide_cursor/0","doc":"","ref":"ExTermbox.Constants.html#t:hide_cursor/0"},{"type":"type","title":"ExTermbox.Constants.input_mode/0","doc":"","ref":"ExTermbox.Constants.html#t:input_mode/0"},{"type":"function","title":"ExTermbox.Constants.input_mode/1","doc":"Retrieves an input mode constant by name","ref":"ExTermbox.Constants.html#input_mode/1"},{"type":"function","title":"Examples - ExTermbox.Constants.input_mode/1","doc":"iex> input_mode(:esc)\n    1\n    iex> input_mode(:esc_with_mouse)\n    5","ref":"ExTermbox.Constants.html#input_mode/1-examples"},{"type":"function","title":"ExTermbox.Constants.input_modes/0","doc":"Retrieves the mapping of input mode constants.","ref":"ExTermbox.Constants.html#input_modes/0"},{"type":"type","title":"ExTermbox.Constants.key/0","doc":"","ref":"ExTermbox.Constants.html#t:key/0"},{"type":"function","title":"ExTermbox.Constants.key/1","doc":"Retrieves a key constant by name","ref":"ExTermbox.Constants.html#key/1"},{"type":"function","title":"Examples - ExTermbox.Constants.key/1","doc":"iex> key(:esc)\n    0x1B\n    iex> key(:space)\n    0x20","ref":"ExTermbox.Constants.html#key/1-examples"},{"type":"function","title":"ExTermbox.Constants.keys/0","doc":"Retrieves the mapping of key constants for use with termbox.\n\nThese are based on terminfo constants. Note that there's some overlap\nof terminfo values. For example, it's not possible to distinguish between\n` ` and `ctrl-h`.","ref":"ExTermbox.Constants.html#keys/0"},{"type":"type","title":"ExTermbox.Constants.output_mode/0","doc":"","ref":"ExTermbox.Constants.html#t:output_mode/0"},{"type":"function","title":"ExTermbox.Constants.output_mode/1","doc":"Retrieves an output mode constant by name","ref":"ExTermbox.Constants.html#output_mode/1"},{"type":"function","title":"Examples - ExTermbox.Constants.output_mode/1","doc":"iex> output_mode(:term_256)\n    2","ref":"ExTermbox.Constants.html#output_mode/1-examples"},{"type":"function","title":"ExTermbox.Constants.output_modes/0","doc":"Retrieves the mapping of output mode constants.","ref":"ExTermbox.Constants.html#output_modes/0"},{"type":"module","title":"ExTermbox.Event","doc":"Represents a termbox event. This can be a key press, click, or window resize.","ref":"ExTermbox.Event.html"},{"type":"module","title":"ExTermbox.EventManager","doc":"This module implements an event manager that notifies subscribers of\nkeyboard, mouse and resize events received from the termbox API.\n\nInternally, the event manager is managing a NIF-based polling routine and\nfanning out polled events to subscribers. It works likes this:\n\n  1. The `ExTermbox.Bindings.start_polling/1` NIF is called with the event\n     manager's pid.\n  2. The NIF creates a background thread for the blocking polling routine and\n     immediately returns with a resource representing a handle for the thread.\n  3. When the polling routine receives an event (e.g., a keypress), it sends\n     a message to the event manager with the event data, and then continues\n     polling for the next event.\n  4. The event manager receives event data from the background thread and\n     notifies all of its subscribers of the event. Steps 3 and 4 are repeated\n     for each event.\n  5. When the event manager is terminated, `ExTermbox.Bindings.stop_polling/0`\n     is called to stop polling and terminate the background thread.\n\nExample Usage:\n\n    def event_loop do\n      receive do\n        {:event, %Event{ch: ?q} = event} ->\n          Bindings.shutdown()\n        {:event, %Event{} = event} ->\n          # handle the event and wait for another...\n          event_loop()\n      end\n    end\n\n    {:ok, pid} = EventManager.start_link()\n    :ok = EventManager.subscribe(self())\n    event_loop()","ref":"ExTermbox.EventManager.html"},{"type":"function","title":"ExTermbox.EventManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExTermbox.EventManager.html#child_spec/1"},{"type":"function","title":"ExTermbox.EventManager.start_link/1","doc":"Starts an event manager process linked to the current process.\n\nRunning multiple instances of the event manager process simultaneously is\ndiscouraged, as it could crash the NIF or cause unexpected behavior. By\ndefault, the process is registered with a fixed name to prevent this.","ref":"ExTermbox.EventManager.html#start_link/1"},{"type":"function","title":"ExTermbox.EventManager.stop/1","doc":"","ref":"ExTermbox.EventManager.html#stop/1"},{"type":"function","title":"ExTermbox.EventManager.subscribe/2","doc":"Subscribes the given subscriber pid to future event notifications.","ref":"ExTermbox.EventManager.html#subscribe/2"},{"type":"module","title":"ExTermbox.Position","doc":"Represents a position on the screen by encoding a pair of cartesian\ncoordinates. The origin is the top-left-most character on the screen\n`(0, 0)`, while x and y increase from left to right and top to bottom,\nrespectively.","ref":"ExTermbox.Position.html"},{"type":"type","title":"ExTermbox.Position.t/0","doc":"","ref":"ExTermbox.Position.html#t:t/0"},{"type":"function","title":"ExTermbox.Position.translate/3","doc":"Translates (shifts) a position by some delta x and y.\n\nReturns a new `%Position{}`.","ref":"ExTermbox.Position.html#translate/3"},{"type":"function","title":"Examples - ExTermbox.Position.translate/3","doc":"iex> translate(%Position{x: 0, y: 0}, 1, 2)\n    %Position{x: 1, y: 2}\n    iex> translate(%Position{x: 10, y: 0}, -1, 0)\n    %Position{x: 9, y: 0}","ref":"ExTermbox.Position.html#translate/3-examples"},{"type":"function","title":"ExTermbox.Position.translate_x/2","doc":"Translates a position by a delta x.\n\nReturns a new `%Position{}`.","ref":"ExTermbox.Position.html#translate_x/2"},{"type":"function","title":"Examples - ExTermbox.Position.translate_x/2","doc":"iex> translate_x(%Position{x: 0, y: 0}, 2)\n    %Position{x: 2, y: 0}\n    iex> translate_x(%Position{x: 2, y: 0}, -1)\n    %Position{x: 1, y: 0}","ref":"ExTermbox.Position.html#translate_x/2-examples"},{"type":"function","title":"ExTermbox.Position.translate_y/2","doc":"Translates a position by a delta y.\n\nReturns a new `%Position{}`.","ref":"ExTermbox.Position.html#translate_y/2"},{"type":"function","title":"Examples - ExTermbox.Position.translate_y/2","doc":"iex> translate_y(%Position{x: 0, y: 0}, 2)\n    %Position{x: 0, y: 2}\n    iex> translate_y(%Position{x: 0, y: 2}, -1)\n    %Position{x: 0, y: 1}","ref":"ExTermbox.Position.html#translate_y/2-examples"},{"type":"module","title":"InlineNif","doc":"An module to make NIF with Elixir escript easier","ref":"InlineNif.html"},{"type":"function","title":"InlineNif.gen_nif_path/2","doc":"Generate a file based on NIF name and its content.","ref":"InlineNif.html#gen_nif_path/2"},{"type":"function","title":"InlineNif.inline/1","doc":"Inject NIF file contents to module attributes","ref":"InlineNif.html#inline/1"},{"type":"exception","title":"InlineNif.NifUndefError","doc":"","ref":"InlineNif.NifUndefError.html"},{"type":"extras","title":"ExTermbox","doc":"# ExTermbox\n\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_termbox.svg)](https://hex.pm/packages/ex_termbox)\n[![Hexdocs.pm](https://img.shields.io/badge/api-hexdocs-brightgreen.svg)](https://hexdocs.pm/ex_termbox)\n[![Travis CI](https://img.shields.io/travis/ndreynolds/ex_termbox/master.svg)](https://travis-ci.org/ndreynolds/ex_termbox)\n\nLow-level [termbox](https://github.com/nsf/termbox) bindings for Elixir.\n\nFor high-level, declarative terminal UIs in Elixir, see\n[Ratatouille](https://github.com/ndreynolds/ratatouille). It builds on top of\nthis library and the termbox API to provide an HTML-like DSL for defining views.\n\nFor the API Reference, see: [https://hexdocs.pm/ex_termbox](https://hexdocs.pm/ex_termbox).","ref":"readme.html"},{"type":"extras","title":"Getting Started - ExTermbox","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Termbox bindings - ExTermbox","doc":"ExTermbox implements the termbox API functions via NIFs:\n\n* [`ExTermbox.Bindings`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html)\n  * [`init/0`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html#init/0)\n  * [`shutdown/0`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html#shutdown/0)\n  * [`width/0`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html#width/0)\n  * [`height/0`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html#height/0)\n  * [`clear/0`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html#clear/0)\n  * [`present/0`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html#present/0)\n  * [`put_cell/1`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html#put_cell/1)\n  * [`change_cell/5`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html#change_cell/5)\n  * [`poll_event/1`](https://hexdocs.pm/ex_termbox/ExTermbox.Bindings.html#poll_event/1)","ref":"readme.html#termbox-bindings"},{"type":"extras","title":"Hello World - ExTermbox","doc":"Let's go through the bundled [hello world example](./examples/hello_world.exs).\nTo follow along, clone this repo and edit the example.\n\nThis repository makes use of [Git submodules](https://git-scm.com/book/en/v2/Git-Tools-Submodules), so make sure you include them in your clone. In recent versions of git, this can be accomplished by including the `--recursive` flag, e.g.\n\n```bash\ngit clone --recursive git@github.com:ndreynolds/ex_termbox.git\n```\n\nWhen the clone is complete, the `c_src/termbox/` directory should have files in it.\n\nYou can also create an\nElixir script in any Mix project with `ex_termbox` in the dependencies list.\nLater, we'll run the example with `mix run  `.\n\nIn a real project, you'll probably want to use an OTP application with a proper\nsupervision tree, but here we'll keep it as simple as possible.\n\nFirst, some aliases for the modules we'll use.\n\n```elixir\nalias ExTermbox.Bindings, as: Termbox\nalias ExTermbox.{Cell, EventManager, Event, Position}\n```\n\nNext, we initialize the termbox application. This initialization should come\nbefore any other termbox functions are called. (Otherwise, your program will\nprobably crash.)\n\n```elixir\n:ok = Termbox.init()\n```\n\nIn order to react to keyboard, click or resize events later, we need to start\nthe event manager and subscribe the current process to any events. The event\nmanager is an abstraction over `poll_event/1` that constantly polls for events\nand notifies its subscribers whenever one is received.\n\n```elixir\n{:ok, _pid} = EventManager.start_link()\n:ok = EventManager.subscribe(self())\n```\n\nTo render content to the screen, we use `put_cell/1`. We pass it\n`%Cell{}` structs that each have a `position` and a `ch`.\n\nThe `position` is a struct representing an (x, y) cartesian coordinate. The\ntop-left-most cell of the screen represents the origin (0, 0).\n\nThe `ch` should be an integer representing the character (e.g., ?a or 97).\nIn the example, we're using charlists for this reason.\n\n```elixir\nfor {ch, x} <- Enum.with_index('Hello, World!') do\n  :ok = Termbox.put_cell(%Cell{position: %Position{x: x, y: 0}, ch: ch})\nend\n\nfor {ch, x} <- Enum.with_index('(Press   to quit)') do\n  :ok = Termbox.put_cell(%Cell{position: %Position{x: x, y: 2}, ch: ch})\nend\n```\n\nSince Elixir is a functional language, it's good practice to avoid this sort of\nimperative style in real applications. Instead, you might build and transform a\ncanvas defined as map or list of cells. Then, when your canvas is ready for\nrendering, it can be synced to termbox via `put_cell/1` in one sweep. This is\nhow the Ratatouille library works.\n\nUntil now, we've only updated termbox's internal buffer. To actually render the\ncontent to the screen, we need to call `present/0`:\n\n```elixir\nTermbox.present()\n```\n\nWhen a key is pressed, it'll be sent to us by the event manager. Once we receive\na 'q' key press, we'll shut down the application.\n\n```elixir\nreceive do\n  {:event, %Event{ch: ?q}} ->\n    :ok = Termbox.shutdown()\nend\n```\n\nYou can use this event-handling logic to respond to events any way you\nlike---e.g., render different content, switch tabs, resize content, etc.\n\nFinally, run the example like this:\n\n```bash\n$ mix run examples/hello_world.exs\n```\n\nYou shuld see the text we rendered and be able to quit with 'q'.","ref":"readme.html#hello-world"},{"type":"extras","title":"Installation - ExTermbox","doc":"","ref":"readme.html#installation"},{"type":"extras","title":"From Hex - ExTermbox","doc":"Add ExTermbox as a dependency in your project's `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_termbox, \"~> 0.3\"}\n  ]\nend\n```\n\nThe Hex package bundles a compatible version of termbox. There are some compile\nhooks to automatically build and link a local copy of `ltermbox` for your\napplication. This should happen the first time you build ExTermbox (e.g., via\n`mix deps.compile`).\n\nSo far the build has been tested on macOS and a few Linux distros. Please add\nan issue if you encounter any problems with the build.","ref":"readme.html#from-hex"},{"type":"extras","title":"From Source - ExTermbox","doc":"To try out the master branch, first clone the repo:\n\n```bash\ngit clone --recurse-submodules https://github.com/ndreynolds/ex_termbox.git\ncd ex_termbox\n```\n\nThe `--recurse-submodules` flag (`--recursive` before Git 2.13) is necessary in\norder to additionally clone the termbox source code, which is required to\nbuild this project.\n\nNext, fetch the deps:\n\n```\nmix deps.get\n```\n\nFinally, try out the included event viewer application:\n\n```\nmix run examples/event_viewer.exs\n```\n\nIf you see the application drawn and can trigger events, you're good to go. Use\n'q' to quit the examples.","ref":"readme.html#from-source"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}